#ifndef SIEVE_ERROR_PRIVATE_H
#define SIEVE_ERROR_PRIVATE_H

#include "sieve-error.h"

/*
 * Types
 */

enum sieve_error_flags {
	SIEVE_ERROR_FLAG_GLOBAL = (1 << 0),
	SIEVE_ERROR_FLAG_GLOBAL_MAX_INFO = (1 << 1),
};

/*
 * Initialization
 */

void sieve_errors_init(struct sieve_instance *svinst);
void sieve_errors_deinit(struct sieve_instance *svinst);

/*
 * Error handler object
 */

struct sieve_error_handler {
	pool_t pool;
	int refcount;

	struct sieve_instance *svinst;

	struct sieve_error_handler *parent;

	unsigned int max_errors;

	unsigned int errors;
	unsigned int warnings;

	/* Should the errorhandler handle or discard info/debug log?
	 * (This does not influence the previous setting)
	 */
	bool log_info;
	bool log_debug;

	void (*logv)(struct sieve_error_handler *ehandler,
		     const struct sieve_error_params *params,
		     enum sieve_error_flags flags,
		     const char *fmt, va_list args) ATTR_FORMAT(4, 0);

	void (*free)(struct sieve_error_handler *ehandler);
};

void sieve_error_handler_init(struct sieve_error_handler *ehandler,
			      struct sieve_instance *svinst, pool_t pool,
			      unsigned int max_errors);

void sieve_error_handler_init_from_parent(struct sieve_error_handler *ehandler,
					  pool_t pool,
					  struct sieve_error_handler *parent);

/*
 * Direct handler calls
 */

void sieve_direct_logv(struct sieve_instance *svinst,
		       struct sieve_error_handler *ehandler,
		       const struct sieve_error_params *params,
		       enum sieve_error_flags flags,
		       const char *fmt, va_list args) ATTR_FORMAT(5, 0);

static inline void ATTR_FORMAT(5, 6)
sieve_direct_log(struct sieve_instance *svinst,
		 struct sieve_error_handler *ehandler,
		 const struct sieve_error_params *params,
		 enum sieve_error_flags flags, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	sieve_direct_logv(svinst, ehandler, params, flags, fmt, args);
	va_end(args);
}

#endif
